"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.mixspaceItemUuidParserConverter = exports.mixspaceItemTypeParser = exports.mixspaceUpdateParamsParser = exports.mixspaceUpdateInputParser = exports.mixspaceCreateResponseParser = exports.mixspaceCreateInputParser = exports.mixspaceParser = exports.mixspaceCoreParser = exports.mixspaceAdminsTraitParser = exports.makeMixspaceAdminsTraitParser = exports.makeMixspaceAdminsParser = exports.mixspaceAdminParser = exports.mixspaceDeleteParamsParser = exports.mixspaceGetOneResultParser = exports.mixspaceGetOneParamsParser = exports.mixspaceListResultParser = exports.mixspaceListQueryParser = void 0;
const zod_1 = require("zod");
const queryParams_1 = require("../common/parsers/queryParams");
const route_constants_1 = require("./route.constants");
const uuid_1 = require("../utils/uuid");
const constants_1 = require("./constants");
const parsers_1 = require("../channel/parsers");
const parsers_id_1 = require("./parsers.id");
const parsers_admin_1 = require("../collective/parsers.admin");
exports.mixspaceListQueryParser = queryParams_1.firestoreListParamsParser
    .merge(zod_1.z
    .object({
    adminId: parsers_admin_1.firebaseAdminIdParser,
})
    .partial())
    .strict();
exports.mixspaceListResultParser = zod_1.z.array(zod_1.z.any()).transform((a) => a);
exports.mixspaceGetOneParamsParser = zod_1.z.object({
    [route_constants_1.MIXSPACE_ID_PARAM_NAME]: parsers_id_1.mixspaceIdParser,
});
exports.mixspaceGetOneResultParser = zod_1.z.any( /* stub */).transform((o) => o);
exports.mixspaceDeleteParamsParser = exports.mixspaceGetOneParamsParser;
exports.mixspaceAdminParser = zod_1.z.string(); // TODO make sense of what we store here
const makeMixspaceAdminsParser = (p) => zod_1.z.array(p);
exports.makeMixspaceAdminsParser = makeMixspaceAdminsParser;
const makeMixspaceAdminsTraitParser = (a) => zod_1.z.object({
    admins: a,
});
exports.makeMixspaceAdminsTraitParser = makeMixspaceAdminsTraitParser;
exports.mixspaceAdminsTraitParser = (0, exports.makeMixspaceAdminsTraitParser)((0, exports.makeMixspaceAdminsParser)(exports.mixspaceAdminParser));
exports.mixspaceCoreParser = exports.mixspaceAdminsTraitParser.merge(zod_1.z.object({
    accessThreshold: zod_1.z.number().default(0),
    image: zod_1.z
        .string()
        .url()
        .refine((s) => s.length > 0, {
        message: 'Field Required',
    }),
    items: zod_1.z.array(zod_1.z.object({
        edge: zod_1.z.boolean().default(false),
        uuid: zod_1.z.string(),
        x: zod_1.z.number(),
        y: zod_1.z.number(),
    })),
    paymentWeights: zod_1.z.object({}),
    reward: zod_1.z.object({
        active: zod_1.z.boolean().default(false),
        assets: zod_1.z.array(zod_1.z.string()),
        price: zod_1.z.number().default(0),
    }),
    title: zod_1.z.string().refine((s) => s.length > 0, {
        message: 'Field Required',
    }),
}));
exports.mixspaceParser = exports.mixspaceCoreParser.merge(zod_1.z.object({
    id: parsers_id_1.mixspaceIdParser,
}));
exports.mixspaceCreateInputParser = exports.mixspaceCoreParser;
exports.mixspaceCreateResponseParser = zod_1.z.any( /*stub*/).transform((a) => a);
exports.mixspaceUpdateInputParser = exports.mixspaceCreateInputParser.partial();
exports.mixspaceUpdateParamsParser = exports.mixspaceGetOneParamsParser;
exports.mixspaceItemTypeParser = zod_1.z.enum(constants_1.MIXSPACE_ITEM_TYPES);
// further parser from item uuid
exports.mixspaceItemUuidParserConverter = zod_1.z.preprocess((uuid) => {
    if (typeof uuid !== 'string')
        throw new Error('uuid must be a string');
    const [type, id] = uuid.split(uuid_1.UUID_SEPARATOR);
    return { type, id };
}, zod_1.z.discriminatedUnion('type', [
    zod_1.z.object({
        type: zod_1.z.literal(constants_1.MIXSPACE_ITEM_TYPE_APPLE),
        id: zod_1.z.string(),
    }),
    zod_1.z.object({
        type: zod_1.z.literal(constants_1.MIXSPACE_ITEM_TYPE_CHANNEL),
        id: parsers_1.channelIdParser,
    }),
]));
