import { ROUTE_IDS } from './constants';
import type { ResetPasswordRequest } from './auth/local/resetPassword/types';
import type { Subset } from './utils/types';
import type { LocalSignupRequest } from './auth/local/signup/types';
import { PayinRouteQuery, PayinRouteResult } from './payin/route/types';
import {
  BlogCreateBody,
  BlogDeleteParams,
  BlogGetOneParams,
  BlogGetOneResult,
  BlogListQuery,
  BlogListResult,
  BlogUpdateBody,
  BlogUpdateParams,
} from './blog/api.types';
import { LocalSigninRequest } from './auth/local/signin/types';
import {
  MixspaceCreateBody,
  MixspaceCreateResult,
  MixspaceDeleteParams,
  MixspaceGetOneParams,
  MixspaceGetOneResult,
  MixspaceListQuery,
  MixspaceListResult,
  MixspaceUpdateBody,
  MixspaceUpdateParams,
} from './mixspace/api.types';
import {
  PayinMethodDeleteParams,
  PayinMethodListQuery,
  PayinMethodListResult,
  PayinMethodUpsertParams,
  PayinMethodUpsertResult,
} from './payinMethod/api.types';
import { CreateBlogResponse } from './blog/parsers';
import { UserId } from './user/types';
import {
  PayoutMethodListQuery,
  PayoutMethodListResult,
  PayoutMethodStripeAuthorizeResult,
  PayoutMethodStripeTokenBody,
} from './payoutMethod/types';

export type RouteId = typeof ROUTE_IDS[number];

interface ApiType<P, Q, B, R> {
  params?: P;
  query?: Q;
  body?: B;
  result: R;
}

interface BodyType<B, R> extends ApiType<unknown, unknown, B, R> {
  body: B;
}

interface ParamsType<P, R> extends ApiType<P, unknown, unknown, R> {
  params: P;
}

interface QueryType<Q, R> extends ApiType<unknown, Q, unknown, R> {
  query: Q;
}

type NoopType = ApiType<unknown, unknown, unknown, null>;

export interface ApiTypes {
  resetPassword: BodyType<ResetPasswordRequest, null>;
  localSignup: BodyType<LocalSignupRequest, UserId>;
  localSignin: BodyType<LocalSigninRequest, UserId>;
  signout: NoopType;
  googleAuthInit: NoopType;
  createBlog: BodyType<BlogCreateBody, CreateBlogResponse>;
  fetchBlog: ParamsType<BlogGetOneParams, BlogGetOneResult>;
  fetchBlogs: QueryType<BlogListQuery, BlogListResult>;
  updateBlog: ApiType<BlogUpdateParams, unknown, BlogUpdateBody, null>;
  deleteBlog: ParamsType<BlogDeleteParams, null>;
  payinRoute: QueryType<PayinRouteQuery, PayinRouteResult>;
  createMixspace: BodyType<MixspaceCreateBody, MixspaceCreateResult>;
  updateMixspace: ApiType<MixspaceUpdateParams, unknown, MixspaceUpdateBody, null>;
  fetchMixspace: ParamsType<MixspaceGetOneParams, MixspaceGetOneResult>;
  fetchMixspaces: QueryType<MixspaceListQuery, MixspaceListResult>;
  deleteMixspace: ParamsType<MixspaceDeleteParams, null>;
  upsertPayinMethod: ParamsType<PayinMethodUpsertParams, PayinMethodUpsertResult>;
  fetchPayinMethods: QueryType<PayinMethodListQuery, PayinMethodListResult>;
  deletePayinMethod: ParamsType<PayinMethodDeleteParams, null>;
  //upsertPayoutMethod: ParamsType<PayoutMethodUpsertParams, PayoutMethodUpsertResult>;
  fetchPayoutMethods: QueryType<PayoutMethodListQuery, PayoutMethodListResult>;
  //deletePayoutMethod: ParamsType<PayoutMethodDeleteParams, null>;
  payoutMethodStripeAuthorize: ApiType<unknown, unknown, unknown, PayoutMethodStripeAuthorizeResult>;
  payoutMethodStripeToken: BodyType<PayoutMethodStripeTokenBody, null>;
}

// eslint-disable-next-line
type AssertApiTypes1 = Subset<RouteId, keyof ApiTypes>;
// eslint-disable-next-line
type AssertApiTypes2 = Subset<keyof ApiTypes, RouteId>;
