import type { z, ZodEffects, ZodType } from 'zod';

// generates is(o)-like method from a parser
// const isStripeCustomerId = isFromParser(stripeCustomerIdParser)
// on practice though, TS doesn't seem to recognize "nested" `is ` statements, therefore usage like
// (id: string): id is StripeCustomerId => isFromParser(stripeCustomerIdParser)(id); is needed
export const isFromParser =
  <T extends ZodType, Input, Output extends Input, P extends ZodEffects<T, Output, Input>>(parser: P) =>
  (o: z.input<P>): o is Output =>
    parser.safeParse(o).success;

// casts a value to a type at the end of parser "chain" i.e. parser.transform(cast<StripePaymentId>())
export const cast =
  <Output>() =>
  (o: any) =>
    o as Output;
